@@ -173,7 +173,6 @@ class Expr(Leaf):
     def clone(self):
         return Expr(self.lhs, self.op, self.rhs, self.negated)
 
-
 class DQ(Leaf):
     def __init__(self, **query):
         super(DQ, self).__init__()
@@ -182,12 +181,21 @@ class DQ(Leaf):
     def clone(self):
         return DQ(**self.query)
 
-
 class Param(Leaf):
     def __init__(self, data):
         self.data = data
         super(Param, self).__init__()
 
+class R(Leaf):
+    def __init__(self, value):
+        self.value = value
+        super(R, self).__init__()
+
+class Ordering(Leaf):
+    def __init__(self, param, asc):
+        self.param = param
+        self.asc = asc
+        super(Ordering, self).__init__()
 
 class Func(Leaf):
     def __init__(self, name, *params):
@@ -220,10 +228,6 @@ class FieldDescriptor(object):
         instance._data[self.att_name] = value
 
 
-Ordering = namedtuple('Ordering', ('param', 'asc'))
-R = namedtuple('R', ('value',))
-
-
 class Field(Leaf):
     _field_counter = 0
     _order = 0
@@ -628,7 +632,7 @@ class QueryCompiler(object):
             clone = expr.clone()
             if not expr._explicit_selection:
                 clone._select = (clone.model_class._meta.primary_key,)
-            subselect, p = self.parse_select_query(clone, max_alias, alias_map)
+            subselect, p = self.generate_select(clone, max_alias, alias_map)
             s = '(%s)' % subselect
         elif isinstance(expr, (list, tuple)):
             exprs = []
@@ -652,12 +656,51 @@ class QueryCompiler(object):
 
         return s, p
 
+    def parse_expr_list(self, s, alias_map):
+        parsed = []
+        data = []
+        for expr in s:
+            expr_str, vars = self.parse_expr(expr, alias_map)
+            parsed.append(expr_str)
+            data.extend(vars)
+        return ', '.join(parsed), data
+
+    def parse_field_dict(self, d):
+        sets, params = [], []
+        for field, expr in d.items():
+            field_str, _ = self.parse_expr(field)
+            # because we don't know whether to call db_value or parse_expr first,
+            # we'd prefer to call parse_expr since its more general, but it does
+            # special things with lists -- it treats them as if it were buliding
+            # up an IN query. for some things we don't want that, so here, if the
+            # expr is *not* a special object, we'll pass thru parse_expr and let
+            # db_value handle it
+            if not isinstance(expr, (Leaf, Model, Query)):
+                expr = Param(expr) # pass through to the fields db_value func
+            val_str, val_params = self.parse_expr(expr)
+            val_params = [field.db_value(vp) for vp in val_params]
+            sets.append((field_str, val_str))
+            params.extend(val_params)
+        return sets, params
+
     def parse_query_node(self, qnode, alias_map):
         if qnode is not None:
             return self.parse_expr(qnode, alias_map)
         return '', []
 
-    def parse_joins(self, joins, model_class, alias_map):
+    def calculate_alias_map(self, query, start=1):
+        alias_map = {query.model_class: 't%s' % start}
+        for model, joins in query._joins.items():
+            if model not in alias_map:
+                start += 1
+                alias_map[model] = 't%s' % start
+            for join in joins:
+                if join.model_class not in alias_map:
+                    start += 1
+                    alias_map[join.model_class] = 't%s' % start
+        return alias_map
+
+    def generate_joins(self, joins, model_class, alias_map):
         parsed = []
         seen = set()
 
@@ -694,28 +737,7 @@ class QueryCompiler(object):
         _traverse(model_class)
         return parsed
 
-    def parse_expr_list(self, s, alias_map):
-        parsed = []
-        data = []
-        for expr in s:
-            expr_str, vars = self.parse_expr(expr, alias_map)
-            parsed.append(expr_str)
-            data.extend(vars)
-        return ', '.join(parsed), data
-
-    def calculate_alias_map(self, query, start=1):
-        alias_map = {query.model_class: 't%s' % start}
-        for model, joins in query._joins.items():
-            if model not in alias_map:
-                start += 1
-                alias_map[model] = 't%s' % start
-            for join in joins:
-                if join.model_class not in alias_map:
-                    start += 1
-                    alias_map[join.model_class] = 't%s' % start
-        return alias_map
-
-    def parse_select_query(self, query, start=1, alias_map=None):
+    def generate_select(self, query, start=1, alias_map=None):
         model = query.model_class
         db = model._meta.database
 
@@ -736,7 +758,7 @@ class QueryCompiler(object):
 
         parts.append('FROM %s AS %s' % (self.quote(model._meta.db_table), alias_map[model]))
 
-        joins = self.parse_joins(query._joins, query.model_class, alias_map)
+        joins = self.generate_joins(query._joins, query.model_class, alias_map)
         if joins:
             parts.append(' '.join(joins))
 
@@ -769,21 +791,11 @@ class QueryCompiler(object):
 
         return ' '.join(parts), params
 
-    def _parse_field_dictionary(self, d):
-        sets, params = [], []
-        for field, expr in d.items():
-            field_str, _ = self.parse_expr(field)
-            val_str, val_params = self.parse_expr(expr)
-            val_params = [field.db_value(vp) for vp in val_params]
-            sets.append((field_str, val_str))
-            params.extend(val_params)
-        return sets, params
-
-    def parse_update_query(self, query):
+    def generate_update(self, query):
         model = query.model_class
 
         parts = ['UPDATE %s SET' % self.quote(model._meta.db_table)]
-        sets, params = self._parse_field_dictionary(query._update)
+        sets, params = self.parse_field_dict(query._update)
 
         parts.append(', '.join('%s=%s' % (f, v) for f, v in sets))
 
@@ -793,18 +805,18 @@ class QueryCompiler(object):
             params.extend(w_params)
         return ' '.join(parts), params
 
-    def parse_insert_query(self, query):
+    def generate_insert(self, query):
         model = query.model_class
 
         parts = ['INSERT INTO %s' % self.quote(model._meta.db_table)]
-        sets, params = self._parse_field_dictionary(query._insert)
+        sets, params = self.parse_field_dict(query._insert)
 
         parts.append('(%s)' % ', '.join(s[0] for s in sets))
         parts.append('VALUES (%s)' % ', '.join(s[1] for s in sets))
 
         return ' '.join(parts), params
 
-    def parse_delete_query(self, query):
+    def generate_delete(self, query):
         model = query.model_class
 
         parts = ['DELETE FROM %s' % self.quote(model._meta.db_table)]
@@ -844,7 +856,7 @@ class QueryCompiler(object):
             parts.append("DEFAULT NEXTVAL('%s')" % self.quote(field.sequence))
         return ' '.join(p % attrs for p in parts)
 
-    def parse_create_table(self, model_class, safe=False):
+    def create_table_sql(self, model_class, safe=False):
         parts = ['CREATE TABLE']
         if safe:
             parts.append('IF NOT EXISTS')
@@ -854,7 +866,7 @@ class QueryCompiler(object):
         return parts
 
     def create_table(self, model_class, safe=False):
-        return ' '.join(self.parse_create_table(model_class, safe))
+        return ' '.join(self.create_table_sql(model_class, safe))
 
     def drop_table(self, model_class, fail_silently=False, cascade=False):
         parts = ['DROP TABLE']
@@ -865,7 +877,7 @@ class QueryCompiler(object):
             parts.append('CASCADE')
         return ' '.join(parts)
 
-    def parse_create_index(self, model_class, fields, unique):
+    def create_index_sql(self, model_class, fields, unique):
         tbl_name = model_class._meta.db_table
         colnames = [f.db_column for f in fields]
         parts = ['CREATE %s' % ('UNIQUE INDEX' if unique else 'INDEX')]
@@ -875,7 +887,7 @@ class QueryCompiler(object):
         return parts
 
     def create_index(self, model_class, fields, unique):
-        return ' '.join(self.parse_create_index(model_class, fields, unique))
+        return ' '.join(self.create_index_sql(model_class, fields, unique))
 
     def create_sequence(self, sequence_name):
         return 'CREATE SEQUENCE %s;' % self.quote(sequence_name)
@@ -1045,6 +1057,10 @@ class Query(object):
         self._joins = {self.model_class: []} # adjacency graph
         self._where = None
 
+    def __repr__(self):
+        sql, params = self.sql()
+        return '%s %s %s' % (self.model_class, sql, params)
+
     def clone(self):
         query = type(self)(self.model_class)
         if self._where is not None:
@@ -1144,8 +1160,8 @@ class Query(object):
             query = query.ensure_join(lm, rm, field_obj)
         return query.where(dq_node)
 
-    def get_compiler(self):
-        return self.database.get_compiler()
+    def compiler(self):
+        return self.database.compiler()
 
     def sql(self):
         raise NotImplementedError
@@ -1323,9 +1339,17 @@ class SelectQuery(Query):
             raise self.model_class.DoesNotExist('instance matching query does not exist:\nSQL: %s\nPARAMS: %s' % (
                 self.sql()
             ))
+    
+    def first(self):
+        res = self.execute()
+        res.fill_cache(1)
+        try:
+            return res._result_cache[0]
+        except IndexError:
+            pass
 
     def sql(self):
-        return self.get_compiler().parse_select_query(self)
+        return self.compiler().generate_select(self)
 
     def verify_naive(self):
         for expr in self._select:
@@ -1381,7 +1405,7 @@ class UpdateQuery(Query):
     join = not_allowed('joining')
 
     def sql(self):
-        return self.get_compiler().parse_update_query(self)
+        return self.compiler().generate_update(self)
 
     def execute(self):
         return self.database.rows_affected(self._execute())
@@ -1403,7 +1427,7 @@ class InsertQuery(Query):
     where = not_allowed('where clause')
 
     def sql(self):
-        return self.get_compiler().parse_insert_query(self)
+        return self.compiler().generate_insert(self)
 
     def execute(self):
         return self.database.last_insert_id(self._execute(), self.model_class)
@@ -1412,7 +1436,7 @@ class DeleteQuery(Query):
     join = not_allowed('joining')
 
     def sql(self):
-        return self.get_compiler().parse_delete_query(self)
+        return self.compiler().generate_delete(self)
 
     def execute(self):
         return self.database.rows_affected(self._execute())
@@ -1497,7 +1521,7 @@ class Database(object):
     def rows_affected(self, cursor):
         return cursor.rowcount
 
-    def get_compiler(self):
+    def compiler(self):
         return self.compiler_class(
             self.quote_char, self.interpolation, self.field_overrides,
             self.op_overrides)
@@ -1527,44 +1551,6 @@ class Database(object):
             self.set_autocommit(self.autocommit)
         return self.__local.autocommit
 
-    def get_tables(self):
-        raise NotImplementedError
-
-    def get_indexes_for_table(self, table):
-        raise NotImplementedError
-
-    def sequence_exists(self, seq):
-        raise NotImplementedError
-
-    def create_table(self, model_class):
-        qc = self.get_compiler()
-        return self.execute_sql(qc.create_table(model_class))
-
-    def create_index(self, model_class, fields, unique=False):
-        qc = self.get_compiler()
-        if not isinstance(fields, (list, tuple)):
-            raise ValueError('fields passed to "create_index" must be a list or tuple: "%s"' % fields)
-        field_objs = [model_class._meta.fields[f] if isinstance(f, basestring) else f for f in fields]
-        return self.execute_sql(qc.create_index(model_class, field_objs, unique))
-
-    def create_foreign_key(self, model_class, field):
-        if not field.primary_key:
-            return self.create_index(model_class, [field], field.unique)
-
-    def create_sequence(self, seq):
-        if self.sequences:
-            qc = self.get_compiler()
-            return self.execute_sql(qc.create_sequence(seq))
-
-    def drop_table(self, model_class, fail_silently=False):
-        qc = self.get_compiler()
-        return self.execute_sql(qc.drop_table(model_class, fail_silently))
-
-    def drop_sequence(self, seq):
-        if self.sequences:
-            qc = self.get_compiler()
-            return self.execute_sql(qc.drop_sequence(seq))
-
     def transaction(self):
         return transaction(self)
 
@@ -1585,6 +1571,44 @@ class Database(object):
                 self.set_autocommit(orig)
         return inner
 
+    def get_tables(self):
+        raise NotImplementedError
+
+    def get_indexes_for_table(self, table):
+        raise NotImplementedError
+
+    def sequence_exists(self, seq):
+        raise NotImplementedError
+
+    def create_table(self, model_class):
+        qc = self.compiler()
+        return self.execute_sql(qc.create_table(model_class))
+
+    def create_index(self, model_class, fields, unique=False):
+        qc = self.compiler()
+        if not isinstance(fields, (list, tuple)):
+            raise ValueError('fields passed to "create_index" must be a list or tuple: "%s"' % fields)
+        field_objs = [model_class._meta.fields[f] if isinstance(f, basestring) else f for f in fields]
+        return self.execute_sql(qc.create_index(model_class, field_objs, unique))
+
+    def create_foreign_key(self, model_class, field):
+        if not field.primary_key:
+            return self.create_index(model_class, [field], field.unique)
+
+    def create_sequence(self, seq):
+        if self.sequences:
+            qc = self.compiler()
+            return self.execute_sql(qc.create_sequence(seq))
+
+    def drop_table(self, model_class, fail_silently=False):
+        qc = self.compiler()
+        return self.execute_sql(qc.drop_table(model_class, fail_silently))
+
+    def drop_sequence(self, seq):
+        if self.sequences:
+            qc = self.compiler()
+            return self.execute_sql(qc.drop_sequence(seq))
+
 
 class SqliteDatabase(Database):
     op_overrides = {
@@ -1699,7 +1723,7 @@ class MySQLDatabase(Database):
         return mysql.connect(db=database, **conn_kwargs)
 
     def create_foreign_key(self, model_class, field):
-        compiler = self.get_compiler()
+        compiler = self.compiler()
         framing = """
             ALTER TABLE %(table)s ADD CONSTRAINT %(constraint)s
             FOREIGN KEY (%(field)s) REFERENCES %(to)s(%(to_field)s)%(cascade)s;
@@ -1752,10 +1776,6 @@ class transaction(object):
 class DoesNotExist(Exception):
     pass
 
-# doing an IN on empty set
-class EmptyResultException(Exception):
-    pass
-
 
 default_database = SqliteDatabase('peewee.db')
 
@@ -2068,7 +2088,3 @@ def sort_models_topologically(models):
     for m in sorted(models, key=names, reverse=True):
         dfs(m)
     return list(reversed(ordering))  # want parents first in output ordering
-
-def raw_sql(query):
-    db = query.model_class._meta.database
-    return query.sql(db.get_compiler())
